import django_filters
from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from netbox_disk.models import Filesystem


class FilesystemFilter(NetBoxModelFilterSet):
    """Filter capabilities for Filesystem instances."""

    class Meta:
        model = Filesystem
        fields = ("filesystem")

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
            Q(filesystem__icontains=value)
        )
        return queryset.filter(qs_filter)


from extras.plugins import PluginMenuButton, PluginMenuItem
from extras.plugins import PluginMenu
from utilities.choices import ButtonColorChoices

disk_menu_item = PluginMenuItem(
    link="plugins:netbox_disk:disk_list",
    link_text="Disks",
    permissions=["netbox_disk.disk_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_disk:disk_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_disk.add_disk"],
        ),
        PluginMenuButton(
            "plugins:netbox_disk:disk_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_disk.add_disk"],
        ),
    ),
)

filesystem_menu_item = PluginMenuItem(
    link="plugins:netbox_disk:filesystem_list",
    link_text="Filesystem",
    permissions=["netbox_disk.disk_view"],
    buttons=(
        PluginMenuButton(
            "plugins:netbox_disk:filesystem_add",
            "Add",
            "mdi mdi-plus-thick",
            ButtonColorChoices.GREEN,
            permissions=["netbox_disk.add_disk"],
        ),
        PluginMenuButton(
            "plugins:netbox_disk:filesystem_import",
            "Import",
            "mdi mdi-upload",
            ButtonColorChoices.CYAN,
            permissions=["netbox_disk.add_disk"],
        ),
    ),
)


menu = PluginMenu(
    label="NetBox Disk",
    groups=(
        (
            "Disk Configuration",
            (
                disk_menu_item,
                filesystem_menu_item
            ),
        ),
    ),
    icon_class="mdi mdi-disc",
)


from netbox.views import generic

from netbox_disk.filters import FilesystemFilter
from netbox_disk.forms import (
    FilesystemImportForm,
    FilesystemFilterForm,
    FilesystemForm,
    FilesystemBulkEditForm
)
from netbox_disk.models import Filesystem
from netbox_disk.tables import FilesystemTable


class FilesystemListView(generic.ObjectListView):
    queryset = Filesystem.objects.all()
    filterset = FilesystemFilter
    filterset_form = FilesystemFilterForm
    table = FilesystemTable


class FilesystemView(generic.ObjectView):
    """Display Filesystem details"""

    queryset = Filesystem.objects.all()


class FilesystemEditView(generic.ObjectEditView):
    """View for editing a Filesystem instance."""

    queryset = Filesystem.objects.all()
    form = FilesystemForm
    default_return_url = "plugins:netbox_disk:filesystem_list"


class FilesystemDeleteView(generic.ObjectDeleteView):
    queryset = Filesystem.objects.all()
    default_return_url = "plugins:netbox_disk:filesystem_list"


class FilesystemBulkImportView(generic.BulkImportView):
    queryset = Filesystem.objects.all()
    model_form = FilesystemImportForm
    table = FilesystemTable
    default_return_url = "plugins:netbox_disk:filesystem_list"


class FilesystemBulkEditView(generic.BulkEditView):
    queryset = Filesystem.objects.all()
    filterset = FilesystemFilter
    table = FilesystemTable
    form = FilesystemBulkEditForm


class FilesystemBulkDeleteView(generic.BulkDeleteView):
    queryset = Filesystem.objects.all()
    table = FilesystemTable



from django.db import models, transaction
from django.urls import reverse

from netbox.models import NetBoxModel
from netbox.search import SearchIndex, register_search


class Disk(NetBoxModel):
    vg_name = models.CharField(
        unique=False,
        max_length=255,
    )
    lv_name = models.CharField(
        unique=False,
        max_length=255,
    )
    size = models.PositiveIntegerField(
        verbose_name="Size (GB)"
    )
    path = models.CharField(
        unique=False,
        max_length=255,
    )
    cluster = models.ForeignKey(
        to='virtualization.Cluster',
        on_delete=models.PROTECT,
        related_name='disks',
    )
    virtual_machine = models.ForeignKey(
        to='virtualization.VirtualMachine',
        on_delete=models.CASCADE,
        related_name='disks',
    )
    description = models.CharField(
        max_length=200,
        blank=True,
    )

    clone_fields = ["vg_name", "lv_name", "size", "path", "virtual_machine", "description"]

    def get_absolute_url(self):
        return reverse("plugins:netbox_disk:disk", kwargs={"pk": self.id})

    def __str__(self):
        return f"VM: test-vm vg_{self.vg_name}-lv_{self.lv_name}"

    class Meta:
        ordering = ("vg_name", "lv_name", "size", "path")


@register_search
class DiskIndex(SearchIndex):
    model = Disk
    fields = (
        ("vg_name", 100),
        ("lv_name", 150),
        ("size", 200),
        ("path", 200),
    )


class Filesystem(NetBoxModel):
    filesystem = models.CharField(
        unique=False,
        max_length=255,
    )
    description = models.CharField(
        max_length=255,
        blank=True,
    )

    clone_fields = ["filesystem", "description"]

    def get_absolute_url(self):
        return reverse("plugins:netbox_disk:filesystem", kwargs={"pk": self.id})

    def __str__(self):
        return f"{self.filesystem}"

    class Meta:
        ordering = ("filesystem", "description")


@register_search
class FilesystemIndex(SearchIndex):
    model = Disk
    fields = (
        ("filesystem", 100)
    )



from django import forms

from django.forms import (
    CharField,
    IntegerField,
    BooleanField,
    NullBooleanField,
)
from django.urls import reverse_lazy

from netbox.forms import (
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
    NetBoxModelForm,
)
from utilities.forms import (
    add_blank_choice,
    BulkEditNullBooleanSelect,
    DynamicModelMultipleChoiceField,
    TagFilterField,
    StaticSelect,
    CSVChoiceField,
    CSVModelChoiceField,
    DynamicModelChoiceField,
    APISelect,
    StaticSelectMultiple,
    add_blank_choice,
)

from netbox_disk.models import Filesystem


class FilesystemForm(NetBoxModelForm):
    """Form for creating a new Filesystem object."""

    class Meta:
        model = Filesystem

        fields = (
            "filesystem",
            "description",
        )


class FilesystemFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering Filesystem instances."""

    model = Filesystem

    filesystem = CharField(
        required=False,
        label="Name",
    )


class FilesystemImportForm(NetBoxModelImportForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class Meta:
        model = Filesystem

        fields = (
            "filesystem",
            "description",
        )


class FilesystemBulkEditForm(NetBoxModelBulkEditForm):
    model = Filesystem

    filesystem = CharField(
        required=False,
        label="Name",
    )
    description = CharField(max_length=255, required=False)

    fieldsets = (
        (
            None,
            ("filesystem", "description"),
        ),
    )
    nullable_fields = ("description")



import django_filters
from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from netbox_disk.models import Disk
from virtualization.models import Cluster, VirtualMachine


class DiskFilter(NetBoxModelFilterSet):
    """Filter capabilities for Disk instances."""
    cluster = django_filters.ModelMultipleChoiceFilter(
        field_name='cluster__name',
        queryset=Cluster.objects.all(),
        to_field_name='name',
        label='Cluster',
    )
    virtual_machine = django_filters.ModelMultipleChoiceFilter(
        field_name='virtual_machine__name',
        queryset=VirtualMachine.objects.all(),
        to_field_name='name',
        label='Virtual machine (name)',
    )

    class Meta:
        model = Disk
        fields = ("vg_name", "lv_name", "size", "path", "cluster", "virtual_machine")

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
            Q(vg_name__icontains=value)
            | Q(lv_name__icontains=value)
            | Q(size__icontains=value)
        )
        return queryset.filter(qs_filter)


from django.urls import path

from netbox.views.generic import ObjectChangeLogView

from netbox_disk.models import Disk, Filesystem
from netbox_disk.views import (
    # disk
    DiskListView,
    DiskView,
    DiskEditView,
    DiskDeleteView,
    DiskBulkImportView,
    DiskBulkEditView,
    DiskBulkDeleteView,
    # filesystem
    FilesystemListView,
    FilesystemView,
    FilesystemEditView,
    FilesystemDeleteView,
    FilesystemBulkImportView,
    FilesystemBulkEditView,
    FilesystemBulkDeleteView,
)

app_name = "netbox_disk"

urlpatterns = [
    #
    # Disk urls
    #
    path("disks/", DiskListView.as_view(), name="disk_list"),
    path("disks/add/", DiskEditView.as_view(), name="disk_add"),
    path("disks/import/", DiskBulkImportView.as_view(), name="disk_import"),
    path("disks/edit/", DiskBulkEditView.as_view(), name="disk_bulk_edit"),
    path("disks/delete/", DiskBulkDeleteView.as_view(), name="disk_bulk_delete"),
    path("disks/<int:pk>/", DiskView.as_view(), name="disk"),
    path("disks/<int:pk>/edit/", DiskEditView.as_view(), name="disk_edit"),
    path("disks/<int:pk>/delete/", DiskDeleteView.as_view(), name="disk_delete"),
    path(
        "disks/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="disk_changelog",
        kwargs={"model": Disk},
    ),
    #
    # Filesystem urls
    #
    path("filesystem/", FilesystemListView.as_view(), name="filesystem_list"),
    path("filesystem/add/", FilesystemEditView.as_view(), name="filesystem_add"),
    path("filesystem/import/", FilesystemBulkImportView.as_view(), name="filesystem_import"),
    path("filesystem/edit/", FilesystemBulkEditView.as_view(), name="filesystem_bulk_edit"),
    path("filesystem/delete/", FilesystemBulkDeleteView.as_view(), name="filesystem_bulk_delete"),
    path("filesystem/<int:pk>/", FilesystemView.as_view(), name="filesystem"),
    path("filesystem/<int:pk>/edit/", FilesystemEditView.as_view(), name="filesystem_edit"),
    path("filesystem/<int:pk>/delete/", FilesystemBulkDeleteView.as_view(), name="filesystem_delete"),
    path(
        "filesystem/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="filesystem_changelog",
        kwargs={"model": Filesystem},
    ),
]


from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.routers import APIRootView

from netbox.api.viewsets import NetBoxModelViewSet

from netbox_disk.api.serializers import (
    DiskSerializer,
    FilesystemSerializer
)
from netbox_disk.filters import DiskFilter, FilesystemFilter
from netbox_disk.models import Disk, Filesystem


class NetboxDiskRootView(APIRootView):
    """
    NetboxDNS API root view
    """

    def get_view_name(self):
        return "NetboxDisk"


class DiskViewSet(NetBoxModelViewSet):
    queryset = Disk.objects.all()
    serializer_class = DiskSerializer
    filterset_class = DiskFilter

    @action(detail=True, methods=["get"])
    def disks(self, request, pk=None):
        disks = Disk.objects.filter(disks__id=pk)
        serializer = DiskSerializer(disks, many=True, context={"request": request})
        return Response(serializer.data)


class FilesystemViewSet(NetBoxModelViewSet):
    queryset = Filesystem.objects.all()
    serializer_class = FilesystemSerializer
    filterset_class = FilesystemFilter

    @action(detail=True, methods=["get"])
    def filesystem(self, request, pk=None):
        filesystem = Filesystem.objects.filter(disks__id=pk)
        serializer = FilesystemSerializer(filesystem, many=True, context={"request": request})
        return Response(serializer.data)



from netbox.api.routers import NetBoxRouter

from netbox_disk.api.views import (
    NetboxDiskRootView,
    DiskViewSet,
    FilesystemViewSet
)

router = NetBoxRouter()
router.APIRootView = NetboxDiskRootView

router.register("disks", DiskViewSet)
router.register("filesystem", FilesystemViewSet)

urlpatterns = router.urls



import django_tables2 as tables

from netbox.tables import (
    NetBoxTable,
    ChoiceFieldColumn,
    ToggleColumn,
    TagColumn,
    ActionsColumn,
)

from netbox_disk.models import Disk


class DiskBaseTable(NetBoxTable):
    """Base class for tables displaying Disks"""

    vg_name = tables.Column(
        linkify=True,
    )
    lv_name = tables.Column()
    size = tables.Column()
    path = tables.Column()
    cluster = tables.Column(
        linkify=True
    )


class DiskTable(DiskBaseTable):
    """Table for displaying Disk objects."""

    pk = ToggleColumn()
    virtual_machine = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = Disk
        fields = (
            "pk",
            "vg_name",
            "lv_name",
            "size",
            "path",
            "cluster",
            "virtual_machine",
            "description",
        )
        default_columns = (
            "virtual_machine",
            "vg_name",
            "lv_name",
            "size",
            "path",
            "cluster",
        )


class RelatedDiskTable(DiskBaseTable):
    actions = ActionsColumn(actions=())

    class Meta(NetBoxTable.Meta):
        model = Disk
        fields = (
            "vg_name",
            "lv_name",
            "size",
            "path",
            "cluster",
        )
        default_columns = (
            "vg_name",
            "lv_name",
            "size",
            "path",
            "cluster",
        )



from django.db.models.functions import Length

from extras.plugins import PluginTemplateExtension

from netbox_disk.models import Disk
from netbox_disk.tables import RelatedDiskTable


class RelatedDisks(PluginTemplateExtension):
    model = "virtualization.virtualmachine"

    def right_page(self):
        obj = self.context.get("object")

        disks = Disk.objects.filter(
            virtual_machine=obj
        )
        disk_table = RelatedDiskTable(
            data=disks
        )

        return self.render(
            "netbox_disk/disk/disk_box.html",
            extra_context={
                "related_disks": disk_table,
            },
        )


template_extensions = [RelatedDisks]



from rest_framework import serializers

from netbox_disk.models import Disk, Filesystem
from virtualization.api.nested_serializers import NestedClusterSerializer, NestedVirtualMachineSerializer


class DiskSerializer(serializers.ModelSerializer):
    cluster = NestedClusterSerializer(required=False, allow_null=True)
    virtual_machine = NestedVirtualMachineSerializer(required=False, allow_null=True)

    class Meta:
        model = Disk
        fields = (
            "id",
            "vg_name",
            "lv_name",
            "size",
            "path",
            "description",
            "cluster",
            "virtual_machine",
            "created",
            "last_updated",
            "custom_fields",
        )


class FilesystemSerializer(serializers.ModelSerializer):

    class Meta:
        model = Filesystem
        fields = (
            "id",
            "filesystem",
            "description",
            "created",
            "last_updated",
            "custom_fields",
        )
